(****************************************************************
 **   WetComm                                                  **
 **   Copyright(C) 2005 Stefan Reiser, Institut fuer Geodaesie **
 **   und Photogrammetrie, TU Braunschweig                     **
 **   2005/02/12                                               **
 ****************************************************************)

unit wcomm1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit, StdCtrls, OoMisc, AdPort, ExtCtrls, StrUtils,
  DateUtils, AdXPort, Buttons, Spin;

{   AdExcept, AdProtcl, AdPStat, AdTapi, ADTrmEmu;}


type
   TDataRecord = (baro, baroE, qnh, qnhE, alti, altiE, humi, humiE, temp1, temp1E, temp2, temp2E, dew, dewE);

  TFormWComm = class(TForm)
    GroupBox1: TGroupBox;
    ButtonRefresh: TButton;
    LabelBARO: TLabel;
    LabelHUMI: TLabel;
    LabelALTI: TLabel;
    LabelQNH: TLabel;
    LabelTEMP1: TLabel;
    LabelTEMP2: TLabel;
    LabelValQNH: TLabel;
    LabelValBARO: TLabel;
    LabelValALTI: TLabel;
    LabelValHUMI: TLabel;
    LabelValTEMP1: TLabel;
    LabelValTEMP2: TLabel;
    LabelTIME: TLabel;
    GroupBox2: TGroupBox;
    SaveDialogDataLog: TSaveDialog;
    ApdComPort: TApdComPort;
    ButtonStartStop: TButton;
    Label1: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    ButtonInfo: TButton;
    Timer: TTimer;
    ShapeStatus: TShape;
    ButtonConfig: TButton;
    LabelDEW: TLabel;
    LabelValDEW: TLabel;
    StaticTextStatus: TStaticText;
    TimerTimeout: TTimer;
    GroupBox3: TGroupBox;
    ButtonChooseAppendFile: TButton;
    LabeledEditFile: TLabeledEdit;
    ButtonChooseNewFile: TButton;
    OpenDialogDataLog: TOpenDialog;
    SpinEditInterval: TSpinEdit;
    procedure ButtonChooseNewFileClick(Sender: TObject);
    procedure ButtonInfoClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure ButtonStartStopClick(Sender: TObject);
    procedure ApdComPortTriggerAvail(CP: TObject; Count: Word);
    procedure ReadData();
    procedure ButtonRefreshClick(Sender: TObject);
    procedure processData(msg : String);
    procedure clearDataRecord();
    procedure showRecord();
    procedure saveRecord();
    procedure ButtonConfigClick(Sender: TObject);
    procedure TimerTimeoutTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure writeFileHeader();
    procedure ButtonChooseAppendFileClick(Sender: TObject);
  private
    { Private-Deklarationen }
    databuf_ : string;
    datalog_file : text;
    flag_file : text;
    datalog_filename : String;
    flag_filename : String;
    datalogging_flag : bool;
    datarecord : array[TDataRecord] of String;
    procedure parseCommandLineParameters;
  public
    { Public-Deklarationen }
  end;

var
  FormWComm: TFormWComm;

implementation

{$R *.dfm}

procedure TFormWComm.ButtonChooseNewFileClick(Sender: TObject);
begin
   if (SaveDialogDataLog.Execute) then
   begin
      datalog_filename := SaveDialogDataLog.FileName;
      ButtonStartStop.Enabled := true;

      // Datei zuweisen...
      AssignFile(datalog_file, datalog_filename);
      Rewrite(datalog_file);    // ...erzeugen bzw. 'ausleeren',
      writeFileHeader(); // Dateikopf schreiben,
      CloseFile(datalog_file);  // ...und wieder schlieen.
      datalogging_flag := true
   end
   else
   begin
      datalogging_flag := false;
      datalog_filename := '';
      ButtonStartStop.Enabled := false
   end;
   LabeledEditFile.Text := datalog_filename;
end;

procedure TFormWComm.ButtonInfoClick(Sender: TObject);
begin
   Application.MessageBox(
      'Datalogging fr Thommen-Wetterstation'#13#10 +
      #13#10 +
      'Copyright '#169' 2005-06 Stefan Reiser, ' +
      'Institut fr Geodsie und Photogrammetrie,'#13#10 +
      'Technische Universitt Braunschweig'#13#10 +
      '_______________________________________________________________________________' +
      #13#10#13#10 +

      'Aufruf:'#13#10 +
      '   wetcomm [port=Portnummer] [file=Dateiname] [int=Intervall] [flag=Semaphore] [-record] '#13#10 +
      ''#13#10 +
      'Portnummer:'#13#10 +
      'Nummer des COM-Ports. Bei port=0 (Standard) wird ein Auswahldialog geffnet.'#13#10 +
      'Standardeinstellungen: 9600 Baud, Parity=None, Data-Bits=8, Stop-Bits=1.'#13#10 +
      #13#10'Dateiname:'#13#10 +
      'In diese Datei wird aufgezeichnet. An bereits existierende Dateien wird angehngt.'#13#10 +
      #13#10'Semaphore:'#13#10 +
      'Zur Prozessberwachung. Eine Datei dieses Namens wird angelegt, wenn eine Messung'#13#10 +
      'erfolgreich aufgezeichnet wurde. Ein berwachender Prozess kann diese Datei dann'#13#10 +
      'lschen oder Manahmen ergreifen, wenn die Datei nicht vorgefunden wird.'#13#10 +
      #13#10'Intervall:' +
      ' Aufzeichnungsintervall in Minuten.'#13#10 +
      'Record:' +
      ' Sofort aufzeichnen (anderenfalls muss manuell gestartet werden).'#13#10 +
      #13#10 +
      'Beispiel: wetcomm port=2 file=c:\wetter.txt int=45 -record flag=c:\wetcomm.flag'
      , 'WetComm Version 2.0.060628', 0);
end;

procedure TFormWComm.writeFileHeader();
var
   curTime, curDate : TDateTime;
   //line : String;
   YYYY, MM, TT, HH, MMIN, SSEC: String;
begin
   (* Anfang einer Wetter-Datei:
   "THOMMEN HM30",1.0,2562
   02.10.02,1 h (95.8 dy),BARO[mbar], QNH[mbar], ALTI[m], HUMI[%rF], Temp1[C], Temp2[C]
   "Init=",#2002-10-02 14:27:50#,9,76,47871,65283,""
   *)
   curTime := Time();  // jetzt
   curDate := Date();  // heute
   YYYY := intToStr(YearOf  (curDate));
   MM   := intToStr(MonthOf (curDate)); if (length(MM) = 1) then MM := '0' + MM;
   TT   := intToStr(DayOf   (curDate)); if (length(TT) = 1) then TT := '0' + TT;
   HH   := intToStr(hourOf  (curTime)); if (length(HH) = 1) then HH := '0' + HH;
   MMIN := intToStr(minuteOf(curTime)); if (length(MMIN) = 1) then MMIN := '0' + MMIN;
   SSEC := intToStr(secondOf(curTime)); if (length(SSEC) = 1) then SSEC := '0' + SSEC;

   WriteLn(datalog_file, '"THOMMEN HM30",1.0,2562');
   WriteLn(datalog_file, TT + '.' + MM+ '.' + rightStr(YYYY, 2) +
                         ',1 h (95.8 dy),BARO[mbar], QNH[mbar], ALTI[m], HUMI[%rF], Temp1[C], Temp2[C]');
   WriteLn(datalog_file, '"Init=",#' +
                         YYYY + '-' + MM + '-' + TT + ' ' + HH + ':' + MMIN + ':' + SSEC +
                         '#,9,76,49151,65283,"DUMMY-HEADER!"');
end;

procedure TFormWComm.ReadData();
{ readbaro, readqnh, readhumid, readtemp1, readtemp2, readalti }
//var
//   baro, qnh, humid, alti, temp1, temp2 : string;
begin
   TimerTimeout.Enabled := false;  // Timeout resetten.
   { anzeigen, dass wir auf Antwort warten...}
   StaticTextStatus.Color := clyellow;
   StaticTextStatus.Font.Color := clgreen;
   StaticTextStatus.Caption := ' warten... ';
   StaticTextStatus.Visible := true;
   StaticTextStatus.Refresh;

   { zunchst in den Remote-Modus wechseln, falls das Messgert uns
    vergessen haben sollte, sonst antwortet es nicht... }
   ApdComPort.output := 'remote'^M;
   sleep(1000);  // lieber ein bisschen warten, man wei ja nie...
   TimerTimeout.Enabled := true;  // Timeout scharfmachen...
   ApdComPort.output := 'readall'^M;  // Befehl absenden: alle Daten ausgeben...

  { // dummy-Daten...
   ApdComPort.output :=
    #9'       BARO 0.7 mbar ' +
      'ALTI -.9 quirx ' +
      'QNH -0.8 gnarf ' +
      'TEMP2 1.2 gulp ' +
      'HUMI --- ' +
      'TEMP1 1 fomp ' +
      'DEW -199 gnilz ' +
      '*250'^M; }
end;

procedure TFormWComm.TimerTimer(Sender: TObject);
begin
   // Messgert abfragen.
   ReadData;
   // Anm.: wenn Datei zugewiesen, dann ist jetzt auch datalogging_flag gesetzt,
   // d.h. die Antwort des Messgerts landet in der Datei...
end;

// loeschen des derzeitigen Records...
procedure TFormWComm.clearDataRecord();
begin
   datarecord[baro]   := '';
   datarecord[baroE]  := '';
   datarecord[qnh]    := '';
   datarecord[qnhE]   := '';
   datarecord[alti]   := '';
   datarecord[altiE]  := '';
   datarecord[humi]   := '';
   datarecord[humiE]  := '';
   datarecord[temp1]  := '';
   datarecord[temp1E] := '';
   datarecord[temp2]  := '';
   datarecord[temp2E] := '';
   datarecord[dew]  := '';
   datarecord[dewE] := '';
end;

// Anzeigen der aktuellen Messung...
procedure TFormWComm.showRecord();
begin
   labelValBARO.Caption  := datarecord[baro ] + ' ' + datarecord[baroE ];
   labelValQNH.Caption   := datarecord[qnh  ] + ' ' + datarecord[qnhE  ];
   labelValALTI.Caption  := datarecord[alti ] + ' ' + datarecord[altiE ];
   labelValHUMI.Caption  := datarecord[humi ] + ' ' + datarecord[humiE ];
   labelValTEMP1.Caption := datarecord[temp1] + ' ' + datarecord[temp1E];
   labelValTEMP2.Caption := datarecord[temp2] + ' ' + datarecord[temp2E];
   labelValDEW.Caption   := datarecord[dew]   + ' ' + datarecord[dewE];
end;

// Speichern der aktuellen Messung
// (sofern Datalogging eingeschaltet = Datei zugewiesen...)
procedure TFormWComm.saveRecord();
var
   curTime, curDate : TDateTime;
   line : String;
   YYYY, MM, TT, HH, MMIN, SSEC: String;
begin
   if not (datalogging_flag) then
   begin
      // showMessage('Abbruch, kein Datalogging');
      exit;  // raus...
   end;

   (* Anfang einer Wetter-Datei:
   "THOMMEN HM30",1.0,2562
   02.10.02,1 h (95.8 dy),BARO[mbar], QNH[mbar], ALTI[m], HUMI[%rF], Temp1[C], Temp2[C]
   "Init=",#2002-10-02 14:27:50#,9,76,47871,65283,""
   #2002-10-22 17:10:23#,984.1,1018.1,285,100.5,14.1,24.2
   #2002-10-22 17:55:36#,983.9,1018.1,287,100.5,13.3,23.5
   #2002-10-22 18:55:36#,983.5,1018.1,291,100.5,13.3,23.2
   *)

   curTime := Time();  // jetzt
   curDate := Date();  // heute
   YYYY := intToStr(YearOf  (curDate));
   MM   := intToStr(MonthOf (curDate)); if (length(MM) = 1) then MM := '0' + MM;
   TT   := intToStr(DayOf   (curDate)); if (length(TT) = 1) then TT := '0' + TT;
   HH   := intToStr(hourOf  (curTime)); if (length(HH) = 1) then HH := '0' + HH;
   MMIN := intToStr(minuteOf(curTime)); if (length(MMIN) = 1) then MMIN := '0' + MMIN;
   SSEC := intToStr(secondOf(curTime)); if (length(SSEC) = 1) then SSEC := '0' + SSEC;

   line := '';

   // 1. aktuelles Datum & Uhrzeit formatieren (ist ok, denn Aufruf erfolgt
   //    unmittelbar nach Eintreffen des Datensatzes...
   // 2. Datenfelder ausgeben
   line := line +
        '#' + YYYY + '-' +
              MM + '-' +
              TT + ' ' +
              HH + ':' +
              MMIN + ':' +
              SSEC +
        '#' + ',' +
        datarecord[baro ] + ',' +
        datarecord[qnh  ] + ',' +
        datarecord[alti ] + ',' +
        datarecord[humi ] + ',' +
        datarecord[temp1] + ',' +
        datarecord[temp2];

   Append(datalog_file);         // Datei zum Anhngen ffnen.
   WriteLn(datalog_file, line);  // Zeile schreiben.
   CloseFile(datalog_file);      // wieder schlieen.

   // Semaphore setzen: Erzeuge Datei flag_filename, damit andere Prozesse
   // sehen knnen, dass eine Messung erfolgreich geschrieben wurde.
   if not (flag_filename = '') then begin
      AssignFile(flag_file, flag_filename);
      Rewrite(flag_file);
      CloseFile(flag_file);
   end;
end;

// Auswerten der Antwort des Messgerts
// Stefans Schnell&Schmutzig-Nachrichten-Parser(tm) :-)
procedure TFormWComm.processData(msg : String);
var
   groesse : string;
   wert : string;
   einheit : string;
   s : string;
   a : string;
   p : integer;
   done : boolean;
begin
   //showMessage(msg);
   clearDataRecord();
   s := trim(msg);  // fhrende Leerzeichen und Zeug entfernen...

   done := false;
   while not (done) do begin

      // alte Werte aus Variablen entfernen!
      groesse := ''; wert := ''; einheit := '';

      // Ende erreicht (*CHK-Block), dann nichts tun (Nachricht auf '' setzen)
      if (leftStr(s, 1) = '*') then begin
         s := '';
         done := true;
      end;

      // GROESSE
      p := pos(' ', s); // nchstes Leerzeichen finden
      if (p > 0) then
      begin
         a := trim(UpperCase(leftStr(s, p-1)));
         // evtl. vorhandene Anfhrungszeichen entfernen...
         if (leftStr(a, 1)  = '"') then a := rightStr(a, length(a)-1);
         if (rightStr(a, 1) = '"') then a := leftStr (a, length(a)-1);
         groesse := trim(a);
         s := trim(rightStr(s, length(s) - p));
      end else done := true;

      // WERT
      p := pos(' ', s); // nchstes Leerzeichen finden
      if (p > 0) then
      begin
         a := trim(leftStr(s, p-1));
         // evtl. vorhandene Anfhrungszeichen entfernen...
         if (leftStr(a, 1)  = '"') then a := rightStr(a, length(a)-1);
         if (rightStr(a, 1) = '"') then a := leftStr (a, length(a)-1);
         wert := trim(a);
         s := trim(rightStr(s, length(s) - p));
      end else done := true;

      { Vorsicht! Wenn eine Gre nicht verfgbar ist, hlt sich das Messgert
      nicht an das Protokoll. Es gibt dann nicht
      GRSSE SPACE WERT SPACE EINHEIT SPACE aus sondern
      GRSSE SPACE --- SPACE. Der Parser msste wegen solcher berraschungen
      nochmal berarbeitet werden (direkt GROESSE-WERT-EINHEIT-Hppchen bilden
      und dann analysieren). Als Workaround wird erstmal davon
      ausgegangen, dass bei Nichtverfgbarkeit '---' und nur dies ausgegeben
      wird und keine weiteren Fallstricke lauern... (aus Angst vor solchen
      resultiert auch der exzessive Einsatz der trim()-Funktion...
      }

      // EINHEIT
      if not (wert = '---') then //fehlerhaften Messwerten folgt keine Einheit.
      begin
      p := pos(' ', s); // nchstes Leerzeichen finden
         if (p > 0) then
         begin
            a := trim(leftStr(s, p-1));
            // evtl. vorhandene Anfhrungszeichen entfernen...
            if (leftStr(a, 1)  = '"') then a := rightStr(a, length(a)-1);
            if (rightStr(a, 1) = '"') then a := leftStr (a, length(a)-1);
            einheit := trim(a);
            s := trim(rightStr(s, length(s) - p));
         end else done := true;
      end
      else begin  // wenn ---, dann unmglichen Zahlwert setzen...
         wert := '-20000';  // = Kompatiblitt zum Thommen-Programm
      end;

     // ----------------------------------------------
     // jetzt: alle 3 Felder gelesen, Daten in Record
     // ablegen...
      if          (groesse = 'BARO' ) then begin
        datarecord[baro]  := wert;
        datarecord[baroE] := einheit;
      end else if (groesse = 'QNH'  ) then begin
        datarecord[qnh]  := wert;
        datarecord[qnhE] := einheit;
      end else if (groesse = 'ALTI' ) then begin
        datarecord[alti]  := wert;
        datarecord[altiE] := einheit;
      end else if (groesse = 'HUMI' ) then begin
        datarecord[humi]  := wert;
        datarecord[humiE] := einheit;
      end else if (groesse = 'TEMP1') then begin
        datarecord[temp1]  := wert;
        datarecord[temp1E] := einheit;
      end else if (groesse = 'TEMP2') then begin
        datarecord[temp2]  := wert;
        datarecord[temp2E] := einheit;
      end else if (groesse = 'DEW') then begin
        datarecord[dew]  := wert;
        datarecord[dewE] := einheit;
      end;
      //ShowMessage('Restlicher String: $' + s + '$');
      //if (done) then ShowMessage('fertig!');
   end;

   // Neue Messung erhalten...
   // prfen, ob wirklich Daten bertragen wurden, sonst nichts tun...
   // alle Felder mssen gefllt sein:
   if not ((datarecord[baro]  = '') OR (datarecord[qnh]   = '') OR
           (datarecord[alti]  = '') OR (datarecord[humi]  = '') OR
           (datarecord[temp1] = '') OR (datarecord[temp2] = '') OR
           (datarecord[dew]   = '')) then
   begin
      TimerTimeout.Enabled := false;  // Timeout entschrfen...
      StaticTextStatus.Visible := true;
      StaticTextStatus.Color := cllime;
      StaticTextStatus.Font.Color := clgreen;
      StaticTextStatus.Caption := ' Status ok. ';
      StaticTextStatus.Refresh;
      showRecord();  // neue Messung zeigen...
      saveRecord();  // neue Messung speichern...
      LabelTime.caption :=  'Stand: ' + DateToStr(Date()) + ', ' + TimeToStr(getTime());
   end;
end;

procedure TFormWComm.ButtonStartStopClick(Sender: TObject);
var
   I, Code: Integer;
begin
   if Timer.Enabled then
      begin
      Timer.Enabled := false;
      SpinEditInterval.Enabled := true;
      LabeledEditFile.Enabled := true;
      ButtonChooseNewFile.Enabled := true;
      ButtonChooseAppendFile.Enabled := true;
      ButtonRefresh.Enabled := true;
      ButtonConfig.Enabled := true;
      ButtonInfo.Enabled := true;
      ButtonStartStop.Caption := 'Aufzeichnung starten';
      ShapeStatus.Brush.Color := clBtnFace;
      end
   else
      begin
      SpinEditInterval.Enabled := false;
      LabeledEditFile.Enabled := false;
      ButtonChooseNewFile.Enabled := false;
      ButtonChooseAppendFile.Enabled := false;      
      ButtonRefresh.Enabled := false;
      ButtonConfig.Enabled := false;
      ButtonInfo.Enabled := false;
      ButtonStartStop.Caption := 'Aufzeichnung unterbrechen';
      ShapeStatus.Brush.Color := clRed;

      { Fehler bei der Konvertierung in Integer? }
      Val(SpinEditInterval.Text, I, Code);
      if Code <> 0 then
      begin
         Application.MessageBox('Fehler: ungltiges Zeit-Intervall, verwende 1 min stattdessen...', 'Info', 0);
         I := 1;
      end;
      Timer.Interval := I*60000;
      Timer.Enabled := true;
      // ersten Aufruf sofort:
      TimerTimer(Sender);
   end;
end;

procedure TFormWComm.ApdComPortTriggerAvail(CP: TObject; Count: Word);
var
  I : Word;
  C : Char;
begin
  for I := 1 to Count do
  begin
    C := ApdComPort.GetChar;
    // evtl. vorhanenen Datenmll entfernen:
    // Nachricht beginnt mit TAB
    if (C = #9) then databuf_ := ''  // Synchronisation
    else if (C = #13) then processData(databuf_)
    else databuf_ := databuf_ + C;
  end;
  //ShowMessage('Got an OnTriggerAvail event for: ' + databuf_);
  // Hoffen wir, dass alles angekommen ist...

end;

procedure TFormWComm.ButtonRefreshClick(Sender: TObject);
begin
   ReadData
end;

procedure TFormWComm.ButtonConfigClick(Sender: TObject);
begin

   if (ApdComPort <> nil) then
   begin
     ComPortOptions.ComPort := ApdComPort;
   end;

   if ComPortOptions.Execute then
   if (ComPortOptions.ComPort <> nil) then
      begin
         { Close and open com ports }
         ApdComPort.Open := False;

         {reassign the new com port properties }
         ApdComPort.Assign(ComPortOptions.ComPort);

         {Ok to open new port/device}
          ApdComPort.Open := True;
      end;
end;

procedure TFormWComm.TimerTimeoutTimer(Sender: TObject);
begin
   // Kam nach 5 sec keine Antwort: Timeout-Meldung
   StaticTextStatus.Visible := true;
   StaticTextStatus.Color := clred;
   StaticTextStatus.Font.Color := clyellow;
   StaticTextStatus.Caption := 'keine Antw.!';
   StaticTextStatus.Refresh;
end;

procedure TFormWComm.FormCreate(Sender: TObject);
begin
   databuf_ := '';  // Globalen Puffer fr Port initialisieren.
   parseCommandLineParameters;
end;

procedure TFormWComm.ParseCommandLineParameters();
var
   s : String;
   i : Integer;
   port : Integer;
   intervall : Integer;
   value, code : Integer;
   record_flag : boolean;
begin
   port := 0;
   flag_filename := '';
   record_flag := false;
   for i := 1 to 10 do begin // max. Anzahl der Parameter derzeit: 10

      s := ParamStr(i);

      // Port auswhlen...
      if (leftStr(s, 5) = 'port=') then begin
         Val(midStr(s,6,length(s)-5), value, code);
         if (code = 0) then port := value;  // manchmal ist Delphi doch richtig scheulich...
         ApdComPort.ComNumber := port;
      end;

      // Aufzeichnungsintervall...
      if (leftStr(s, 4) = 'int=') then begin
         Val(midStr(s,5,length(s)-4), value, code);
         intervall := 60;
         if (code = 0) then intervall := value;
         SpinEditInterval.value := intervall;
      end;

      // Semaphore benutzen...
      // die angegebene Datei wird erzeugt, sobald eine erfolgreiche Messung
      // durchgefhrt wurde...
      if (leftStr(s, 5) = 'flag=') then begin
         flag_filename := midStr(s, 6, Length(s) - 5);
      end;

      // Ausgabedatei festlegen
      if (leftStr(s, 5) = 'file=') then begin

         // Aufzeichnung direkt starten?
         if FindCmdLineSwitch('record') then record_flag := true;


         datalog_filename := midStr(s, 6, Length(s) - 5);

         // Datei zuweisen...
         AssignFile(datalog_file, datalog_filename);

         // existiert die Datei? sonst erzeugen...
         if (not FileExists(datalog_filename)) then begin
            datalogging_flag := true ;
            Rewrite(datalog_file);    // ...erzeugen bzw. 'ausleeren',
            writeFileHeader(); // Dateikopf schreiben,
            CloseFile(datalog_file);  // ...und wieder schlieen.
         end else begin
            datalogging_flag := true;
            Append(datalog_file);    // ...ffnen
            CloseFile(datalog_file);  // ...und wieder schlieen.
            // ==> Fehler provozieren, wenn Datei ungltig.
         end;
         LabeledEditFile.Text := datalog_filename;
         ButtonStartStop.enabled := true;
      end;

   end;

   if record_flag then ButtonStartStopClick(nil);


//   Str(Port, s);
//   Application.MessageBox(PChar('Port: ' + s + ' File: ' + datalog_filename), 'Aufruf-Parameter');
end;

procedure TFormWComm.ButtonChooseAppendFileClick(Sender: TObject);
begin
   if (OpenDialogDataLog.Execute) then
   begin
      datalog_filename := OpenDialogDataLog.FileName;
      ButtonStartStop.Enabled := true;

      // Datei zuweisen...
      AssignFile(datalog_file, datalog_filename);
      Append(datalog_file);    // ...ffnen
      CloseFile(datalog_file);  // ...und wieder schlieen.
       //... warum eigentlich?? egal, macht keinen Schaden, Fehler werden sofort sichtbar...
      datalogging_flag := true
   end
   else
   begin
      datalogging_flag := false;
      datalog_filename := '';
      ButtonStartStop.Enabled := false
   end;
   LabeledEditFile.Text := datalog_filename;
end;

end.
